<?php

class Kama_Thumbnail {

	public static $opt_name = 'kama_thumbnail';
	public static $opt;

	function __get( $name ){
		if( $name == 'opt' ) return self::$opt;
	}

	function __construct(){
		self::$opt = (object) ( ($tmp = get_option(self::$opt_name)) ? $tmp : self::def_options() );

		if( ! @ self::$opt->no_photo_url )     self::$opt->no_photo_url     = KT_URL .'no_photo.jpg';
		if( ! @ self::$opt->cache_folder )     self::$opt->cache_folder     = str_replace('\\', '/', WP_CONTENT_DIR . '/cache/thumb');
		if( ! @ self::$opt->cache_folder_url ) self::$opt->cache_folder_url = content_url() .'/cache/thumb';

		// allow_hosts
		$ah = & self::$opt->allow_hosts;
		if( $ah && ! is_array($ah) ){
			$ah = preg_split('/[\s,]+/s', trim( $ah ) ); // сделаем массив
			foreach( $ah as & $host ) $host = str_replace('www.', '', $host );
		}
		else
			$ah = array();

		$this->wp_init();
	}

	function wp_init(){
		// админка
		if( is_admin() && ! defined('DOING_AJAX') ){
			add_action('admin_menu', array( & $this, 'admin_options') ); // закомментируйте, чтобы убрать опции из админки
			add_action('admin_menu', array( & $this, 'claer_handler') );

			add_filter('save_post', array( & $this, 'clear_post_meta') );

			// ссылка на настойки со страницы плагинов
			add_filter('plugin_action_links', array( & $this, 'setting_page_link'), 10, 2 );
		}

		if( self::$opt->use_in_content ){
			add_filter('the_content', array( & $this, 'replece_in_content') );
			add_filter('the_content_rss', array( & $this, 'replece_in_content') );
		}

	}

	public static function def_options(){
		return array(
			'meta_key'       => 'photo_URL',
			'cache_folder'   => '', // полный путь до папки миниатюр
			'cache_folder_url' => '', // URL до папки миниатюр
			'no_photo_url'   => '', // УРЛ на заглушку
			'use_in_content' => 'mini',  // искать ли класс mini у картинок в тексте, чтобы изменить их размер
			'auto_clear'     => 0,
			'no_stub'        => 0,
			'quality'        => 85,
			'subdomen'       => '', // поддомены на котором могут быть исходные картинки (через запятую): img.site.ru,img2.site.ru
			'allow_hosts'    => '', // доступные хосты, кроме родного, через запятую
			'debug'          => 0,  // режим дебаг
		);
	}

	## Функции поиска и замены в посте
	function replece_in_content( $content ){
		$miniclass = (self::$opt->use_in_content == 1) ? 'mini' : self::$opt->use_in_content;

		if( false !== strpos( $content, '<img ') && false !== strpos( $content, $miniclass) ){
			$img_ex = '<img([^>]*class=["\'][^\'"]*(?<=[\s\'"])'. $miniclass .'(?=[\s\'"])[^\'"]*[\'"][^>]*)>';
			// разделение ускоряет поиск почти в 10 раз
			$content = preg_replace_callback("~(<a[^>]+>\s*)$img_ex|$img_ex~", array( & $this, '_replece_in_content'), $content );
		}

		return $content;
	}
	function _replece_in_content( $m ){
		$a_prefix = $m[1];
		$is_a_img = '<a' === substr( $a_prefix, 0, 2);
		$attr = $is_a_img ? $m[2] : $m[3];

		$attr = trim( $attr, '/ ');

		// get src="xxx"
		preg_match('~src=[\'"]([^\'"]+)[\'"]~', $attr, $match_src );
		$src = $match_src[1];
		$attr = str_replace( $match_src[0], '', $attr );

		// make args from attrs
		$args = preg_split('~ *(?<!=)["\'] *~', $attr );
		$args = array_filter( $args );

		$_args = array();
		foreach( $args as $val ){
			list( $k, $v ) = preg_split('~=[\'"]~', $val );
			$_args[ $k ] = $v;
		}
		$args = $_args;

		// parse srcset if set
		if( isset($args['srcset']) ){
			$srcsets = array_map('trim', explode(',', $args['srcset']) );
			$_cursize = 0;
			foreach( $srcsets as $_src ){
				preg_match('/ ([0-9]+[a-z]+)$/', $_src, $mm );
				$size = $mm[1];
				$_src = str_replace( $mm[0], '', $_src );

				// retina
				if( $size === '2x' ){
					$src = $_src;
					break;
				}

				$size = intval($size);
				if( $size > $_cursize )
					$src = $_src;

				$_cursize = $size;
			}

			unset( $args['srcset'] );
		}

		//print_r($args + [$src]); echo "\n\n\n\n";
		$kt = new Kama_Make_Thumb( $args, $src );

		return $is_a_img ? $a_prefix . $kt->img() : $kt->a_img();
	}

	## Удалет произвольное поле со ссылкой при обновлении поста, чтобы создать его снова
	function clear_post_meta( $post_id ){
		update_post_meta( $post_id, self::$opt->meta_key, '' );
	}


	### ADMIN PART -----------------------------------
	static function activation(){
		if( ! get_option(self::$opt_name) )
			update_option( self::$opt_name, self::def_options() );
	}

	function uninstall(){
		$this->clear_cache();
		$this->del_customs();

		delete_option( self::$opt_name );
		@ rmdir( $this->opt->cache_folder );
	}

	## для вывода сообещний в админке
	static function show_message( $text = '', $class = 'updated' ){
		$echo = '<div id="message" class="'. $class .' notice is-dismissible"><p>'. $text .'</p></div>';
		add_action('admin_notices', create_function('', "echo '". str_replace("'", "\'", $echo) ."';" ) );
	}

	function admin_options(){
		// Добавляем блок опций на базовую страницу "Чтение"
		add_settings_section('kama_thumbnail', __('Kama Thumbnail Settings','kama-thumbnail'), '', 'media' );

		// Добавляем поля опций. Указываем название, описание,
		// функцию выводящую html код поля опции.
		add_settings_field( 'kt_options_field',
			'
			<p><a class="button" target="_blank" href="'. add_query_arg('kt_clear', 'clear_cache_stub') .'">'. __('Clear nophoto Cache','kama-thumbnail') .'</a></p>
			<p><a class="button" target="_blank" href="'. add_query_arg('kt_clear', 'clear_cache') .'">'. __('Clear all images','kama-thumbnail') .'</a></p>
			<p><a class="button" target="_blank" href="'. add_query_arg('kt_clear', 'del_customs') .'">'. __('Remove Releted Custom Fields','kama-thumbnail') .'</a></p>
			<p><a class="button" target="_blank" href="'. add_query_arg('kt_clear', 'clear_all_data') .'">'. __('Remove all data','kama-thumbnail') .'</a></p>
			',
			array( & $this, 'options_field'), // можно указать ''
			'media', // страница
			'kama_thumbnail' // секция
		);

		// Регистрируем опции, чтобы они сохранялись при отправке
		// $_POST параметров и чтобы callback функции опций выводили их значение.
		register_setting('media', self::$opt_name, array( __CLASS__, 'sanitize_options') );
	}

	## sanitize options
	static function sanitize_options( $opts ){
		foreach( $opts as $key => & $val ){
			if( $key === 'allow_hosts' ){
				$ah = preg_split('/[\s,]+/s', trim( $val ) ); // сделаем массив
				foreach( $ah as & $host ){
					$host = sanitize_text_field( $host );
					$host = str_replace('www.', '', $host );
				}
				$val = implode("\n", $ah );
			}
			else
				$val = sanitize_text_field( $val );
		}

		return $opts;
	}

	function options_field(){
		$opt_name = self::$opt_name;
		$opt = (object) get_option( $opt_name );

		$def_opt = (object) self::def_options();

		$out = '';

		$out .= '
		<input type="text" name="'. $opt_name .'[cache_folder]" value="'. $opt->cache_folder .'" style="width:100%;" placeholder="'. $this->opt->cache_folder .'"><br>
		'. __('Full path to the cache folder with 755 rights or above.','kama-thumbnail') .'
		<br><br>

		<input type="text" name="'. $opt_name .'[cache_folder_url]" value="'. $opt->cache_folder_url .'" style="width:100%;" placeholder="'. $this->opt->cache_folder_url .'"><br>
		'. __('URL of cache folder.','kama-thumbnail') .'
		<br><br>

		<input type="text" name="'. $opt_name .'[no_photo_url]" value="'. $opt->no_photo_url .'" style="width:100%;" placeholder="'. $this->opt->no_photo_url .'"><br>
		'. __('URL of stub image.','kama-thumbnail') .'
		<br><br>

		<input type="text" name="'. $opt_name .'[meta_key]" value="'. $opt->meta_key .'" class="regular-text"><br>
		'. __('Custom field key, where the thumb URL will be. Default:','kama-thumbnail') .' <code>'. $def_opt->meta_key .'</code>
		<br><br>

		<textarea name="'. $opt_name .'[allow_hosts]" style="width:100%;height:45px;">'. esc_textarea($opt->allow_hosts) .'</textarea><br>
		'. __('Hosts from which thumbs can be created. One per line: <i>sub.mysite.com</i>. Specify <code>any</code>, to use any hosts.','kama-thumbnail') .'
		<br><br>

		<input type="text" name="'. $opt_name .'[quality]" value="'. $opt->quality .'" style="width:50px;">
		'. __('Quality of creating thumbs from 0 to 100. Default:','kama-thumbnail') .' <code>'. $def_opt->quality .'</code>
		<br><br>

		<label>
			<input type="checkbox" name="'. $opt_name .'[no_stub]" value="1" '. checked(1, @ $opt->no_stub, 0) .'>
			'. __('Don\'t show nophoto image.','kama-thumbnail') .'
		</label>
		<br><br>

		<label>
			<input type="checkbox" name="'. $opt_name .'[auto_clear]" value="1" '. checked(1, @ $opt->auto_clear, 0) .'>
			'. __('Clear all cache automaticaly every 7 days.','kama-thumbnail') .'
		</label>
		<br><br>

		<label>
			<input type="text" name="'. $opt_name .'[use_in_content]" value="'.( isset($opt->use_in_content) ? esc_attr($opt->use_in_content) : 'mini' ).'">
			'. sprintf( __('Find specified here class of IMG tag in content and make thumb from found image by it sizes. Leave this field empty to disable this function. Default: %s','kama-thumbnail'), '<code>mini</code>' ) .'
		</label>
		<br><br>

		<label>
			<input type="checkbox" name="'. $opt_name .'[debug]" value="1" '. checked(1, @ $opt->debug, 0) .'>
			'. __('Debug mode. Recreates thumbs all time...','kama-thumbnail') .'
		</label>

		';

		echo $out;
	}

	function setting_page_link( $actions, $plugin_file ){
		if( false === strpos( $plugin_file, basename(KT_PATH) ) ) return $actions;

		$settings_link = '<a href="'. admin_url('options-media.php') .'">'. __('Settings','kama-thumbnail') .'</a>';
		array_unshift( $actions, $settings_link );

		return $actions;
	}
	### / ADMIN PART -----------------------------------


	### CLEAR -----------------------------------
	function claer_handler(){
		if( isset($_GET['kt_clear']) && current_user_can('manage_options') ){
			$this->force_clear( $_GET['kt_clear'] );
			ob_start();
			do_action('admin_notices');
			$msg = ob_get_clean();
			wp_die( $msg );
		}

		if( isset( self::$opt->auto_clear ) ) $this->clear();

		$this->clear_stub();
	}

	function clear(){
		$cache_dir = self::$opt->cache_folder;
		$expire_time = time() + ( 3600 * 24 * 7 );

		$expire = @ file_get_contents( $cache_dir .'/expire');

		if( $expire && (int) $expire < time() )	$this->clear_cache();

		@ file_put_contents( $cache_dir .'/expire', $expire_time );

		return;
	}

	function clear_stub(){
		$cache_dir = self::$opt->cache_folder;
		$expire_time = time() + ( 3600 * 24 ); // очистка заглушек каждый день
		//$expire_time = time() + 20; // тест

		$expire = @ file_get_contents( $cache_dir .'/expire_stub');
		if( $expire && (int) $expire < time() )
			$this->clear_cache('only_stub');

		@ file_put_contents( $cache_dir .'/expire_stub', $expire_time );
	}

	## ?kt_clear=clear_cache - очистит кеш картинок ?kt_clear=del_customs - удалит произвольные поля
	function force_clear( $do ){
		switch( $do ){
			case 'clear_cache_stub':
				$text = $this->clear_cache('only_stub');
				break;
			case 'clear_cache':
				$text = $this->clear_cache();
				break;
			case 'del_customs':
				$text = $this->del_customs();
				break;
			case 'clear_all_data':
				$text = $this->clear_cache();
				$text = $this->del_customs();
				break;
		}
	}

	function clear_cache( $only_stub = '' ){
		if( ! $cache_dir = self::$opt->cache_folder ){
			self::show_message( __('Path to cache not set.','kama-thumbnail'), 'error');
			return false;
		}

		if( ! is_dir($cache_dir) ) return true;

		$find = $only_stub ? "$cache_dir/stub_*" : "$cache_dir/*";

		foreach( glob($find) as $obj ) unlink($obj);

		if( $only_stub )
			self::show_message( __('All nophoto thumbs was deleted from <b>Kama Thumbnail</b> cache.','kama-thumbnail') );
		else
			self::show_message( __('<b>Kama Thumbnail</b> cache has been cleared.','kama-thumbnail') );
	}

	function del_customs(){
		global $wpdb;
		if( ! self::$opt->meta_key )
			return self::show_message('meta_key option not set.', 'error');

		if( $wpdb->delete( $wpdb->postmeta, array('meta_key'=>self::$opt->meta_key ) ) )
			self::show_message( sprintf( __('All custom fields <code>%s</code> was deleted.','kama-thumbnail'), self::$opt->meta_key ) );
		else
			self::show_message( sprintf( __('Couldn\'t delete <code>%s</code> custom fields','kama-thumbnail'), self::$opt->meta_key ) );
	}
	### / CLEAR -----------------------------------

}


